==> workbench_approver-1.0.0.tar.gz: workbench_approver/src/Services/ApproverStateTransitionValidation.php <==
<?php

namespace Drupal\workbench_approver\Services;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\content_moderation\StateTransitionValidation;
use Drupal\content_moderation\StateTransitionValidationInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\NodeInterface;

/**
 * Decorates the `content_moderation.state_transition_validation` handler.
 *
 * This service lets us enforce our own rules.
 */
class ApproverStateTransitionValidation extends StateTransitionValidation implements StateTransitionValidationInterface {

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * The approver manager service.
   *
   * @var \Drupal\workbench_approver\Services\ApproverManagerInterface
   */
  protected $approverManager;

  /**
   * Constructs a new StateTransitionValidation.
   *
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_info
   *   The moderation information service.
   */
  public function __construct(ModerationInformationInterface $moderation_info) {
    parent::__construct($moderation_info);
    $this->moderationInfo = $moderation_info;
  }

  /**
   * {@inheritdoc}
   */
  public function getValidTransitions(ContentEntityInterface $entity, AccountInterface $user): array {
    $this->getApproverManager();
    $allowed_transitions = [];
    $transitions = parent::getValidTransitions($entity, $user);
    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    $current_state = (bool) $entity->get('moderation_state')->getString()
      ? $workflow->getTypePlugin()->getState($entity->get('moderation_state')->getString())
      : $workflow->getTypePlugin()->getInitialState();

    // If there are no pending, required reviews, we do nothing.
    if ($entity->isNew() || !$this->approverManager->isBlocked($entity)) {
      return $transitions;
    }

    // We only allow transitions that have a `toStateId` equal to the state id.
    foreach ($transitions as $id => $transition) {
      if ($transition->to()->id() === $current_state->id()) {
        $allowed_transitions[$id] = $transition;
      }
    }

    return $allowed_transitions;
  }

  /**
   * Gets the manager service to avoid a circular reference error.
   *
   * @return void
   *   No return value. Load properties to the class.
   */
  private function getApproverManager(): void {
    // @todo Trying to inject this service throws a circular error.
    // @phpstan-ignore-next-line
    $this->approverManager = \Drupal::service('workbench_approver.manager');
  }

  /**
   * Gets a list of self-to-self transitions.
   *
   * We allow collaboration on content with approvers are assigned. Under this
   * condition, the state of a draft cannot be transitioned to a new state
   * until approvals are met.
   *
   * This method, unlike getValidTransitions(), lets the caller know if
   * a state-to-state transition exists, which allows reviewers to be
   * assigned.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be transitioned.
   *
   * @return bool
   *   True, if the entity's current state has a self-transition.
   */
  public function isApprovable(ContentEntityInterface $entity): bool {
    $return = FALSE;

    // New content and published revisions never need approval.
    assert($entity instanceof NodeInterface);
    if ($entity->isPublished() || $entity->isNew()) {
      return $return;
    }

    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    $current_state = (bool) $entity->get('moderation_state')->getString()
      ? $workflow->getTypePlugin()->getState($entity->get('moderation_state')->getString())
      : $workflow->getTypePlugin()->getInitialState();
    $transitions = $workflow->getTypePlugin()->getTransitionsForState($current_state->id());

    // Is there a transition that has a `toStateId` equal to the current id.
    foreach ($transitions as $transition) {
      if ($transition->to()->id() === $current_state->id()) {
        $return = TRUE;
      }
    }

    return $return;

  }

}

==> workbench_approver-1.0.1.tar.gz: workbench_approver/src/Services/ApproverStateTransitionValidation.php <==
<?php

namespace Drupal\workbench_approver\Services;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\content_moderation\StateTransitionValidationInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\NodeInterface;
use Drupal\workflows\StateInterface;
use Drupal\workflows\WorkflowInterface;

/**
 * Decorates the `content_moderation.state_transition_validation` handler.
 *
 * This service lets us enforce our own rules.
 */
class ApproverStateTransitionValidation implements StateTransitionValidationInterface {

  /**
   * Original service.
   *
   * @var \Drupal\content_moderation\StateTransitionValidationInterface
   */
  protected $original;

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * The approver manager service.
   *
   * @var \Drupal\workbench_approver\Services\ApproverManagerInterface
   */
  protected $approverManager;

  /**
   * Constructs a new StateTransitionValidation.
   *
   * @param \Drupal\content_moderation\StateTransitionValidationInterface $original
   *   The original service being decorated.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_info
   *   The moderation information service.
   */
  public function __construct(StateTransitionValidationInterface $original, ModerationInformationInterface $moderation_info) {
    $this->original = $original;
    $this->moderationInfo = $moderation_info;
  }

  /**
   * {@inheritdoc}
   */
  public function isTransitionValid(WorkflowInterface $workflow, StateInterface $original_state, StateInterface $new_state, AccountInterface $user, ContentEntityInterface $entity) {
    return $this->original->isTransitionValid($workflow, $original_state, $new_state, $user, $entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getValidTransitions(ContentEntityInterface $entity, AccountInterface $user): array {
    $this->getApproverManager();
    $allowed_transitions = [];
    $transitions = $this->original->getValidTransitions($entity, $user);
    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    $current_state = (bool) $entity->get('moderation_state')->getString()
      ? $workflow->getTypePlugin()->getState($entity->get('moderation_state')->getString())
      : $workflow->getTypePlugin()->getInitialState();

    // If there are no pending, required reviews, we do nothing.
    if ($entity->isNew() || !$this->approverManager->isBlocked($entity)) {
      return $transitions;
    }

    // We only allow transitions that have a `toStateId` equal to the state id.
    foreach ($transitions as $id => $transition) {
      if ($transition->to()->id() === $current_state->id()) {
        $allowed_transitions[$id] = $transition;
      }
    }

    return $allowed_transitions;
  }

  /**
   * Gets the manager service to avoid a circular reference error.
   *
   * @return void
   *   No return value. Load properties to the class.
   */
  private function getApproverManager(): void {
    // @todo Trying to inject this service throws a circular error.
    // @phpstan-ignore-next-line
    $this->approverManager = \Drupal::service('workbench_approver.manager');
  }

  /**
   * Gets a list of self-to-self transitions.
   *
   * We allow collaboration on content with approvers are assigned. Under this
   * condition, the state of a draft cannot be transitioned to a new state
   * until approvals are met.
   *
   * This method, unlike getValidTransitions(), lets the caller know if
   * a state-to-state transition exists, which allows reviewers to be
   * assigned.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be transitioned.
   *
   * @return bool
   *   True, if the entity's current state has a self-transition.
   */
  public function isApprovable(ContentEntityInterface $entity): bool {
    $return = FALSE;

    // New content and published revisions never need approval.
    assert($entity instanceof NodeInterface);
    if ($entity->isPublished() || $entity->isNew()) {
      return $return;
    }

    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    $current_state = (bool) $entity->get('moderation_state')->getString()
      ? $workflow->getTypePlugin()->getState($entity->get('moderation_state')->getString())
      : $workflow->getTypePlugin()->getInitialState();
    $transitions = $workflow->getTypePlugin()->getTransitionsForState($current_state->id());

    // Is there a transition that has a `toStateId` equal to the current id.
    foreach ($transitions as $transition) {
      if ($transition->to()->id() === $current_state->id()) {
        $return = TRUE;
      }
    }

    return $return;
  }

}
